@@ -7,14 +7,15 @@
 # This software is licensed as described in the file COPYING, which
 # you should have received as part of this distribution.
 
-import sys
+import base64
 import quopri
+import sys
 
 import jsonpickle.util as util
 import jsonpickle.tags as tags
 import jsonpickle.handlers as handlers
 
-from jsonpickle.compat import set
+from jsonpickle.compat import numeric_types, set, unicode
 from jsonpickle.backend import JSONBackend
 
 
@@ -128,8 +129,10 @@ class Unpickler(object):
         self._proxies = []
 
     def _restore(self, obj):
-        if has_tag(obj, tags.BYTES):
-            restore = self._restore_bytestring
+        if has_tag(obj, tags.B64):
+            restore = self._restore_base64
+        elif has_tag(obj, tags.BYTES):  # Backwards compatibility
+            restore = self._restore_quopri
         elif has_tag(obj, tags.ID):
             restore = self._restore_id
         elif has_tag(obj, tags.REF):  # Backwards compatibility
@@ -158,7 +161,11 @@ class Unpickler(object):
             restore = lambda x: x
         return restore(obj)
 
-    def _restore_bytestring(self, obj):
+    def _restore_base64(self, obj):
+        return base64.decodestring(obj[tags.B64].encode('utf-8'))
+
+    #: For backwards compatibility with bytes data produced by older versions
+    def _restore_quopri(self, obj):
         return quopri.decodestring(obj[tags.BYTES].encode('utf-8'))
 
     def _restore_iterator(self, obj):
@@ -204,6 +211,7 @@ class Unpickler(object):
             for k, v in dictitems:
                 stage1.__setitem__(k, v)
 
+        self._mkref(stage1)
         return stage1
 
     def _restore_id(self, obj):
@@ -315,7 +323,11 @@ class Unpickler(object):
             # ignore the reserved attribute
             if ignorereserved and k in tags.RESERVED:
                 continue
-            self._namestack.append(k)
+            if isinstance(k, numeric_types):
+                str_k = unicode(k)
+            else:
+                str_k = k
+            self._namestack.append(str_k)
             k = restore_key(k)
             # step into the namespace
             value = self._restore(v)
@@ -398,7 +410,11 @@ class Unpickler(object):
         data = {}
         restore_key = self._restore_key_fn()
         for k, v in sorted(obj.items(), key=util.itemgetter):
-            self._namestack.append(k)
+            if isinstance(k, numeric_types):
+                str_k = unicode(k)
+            else:
+                str_k = k
+            self._namestack.append(str_k)
             k = restore_key(k)
             data[k] = self._restore(v)
             self._namestack.pop()
